@@ -4,31 +4,38 @@ import genshi
 import datetime
 from urllib import urlencode
 
-from ckan.lib.base import BaseController, c, model, request, render, h, g
-from ckan.lib.base import ValidationException, abort, gettext
-from pylons.i18n import get_lang, _
-from ckan.lib.helpers import Page
+from pylons.i18n import get_lang
+
+import ckan.lib.base as base
+import ckan.lib.helpers as h
 import ckan.lib.maintain as maintain
-from ckan.lib.navl.dictization_functions import DataError, unflatten, validate
-from ckan.logic import NotFound, NotAuthorized, ValidationError
-from ckan.logic import check_access, get_action
-from ckan.logic import tuplize_dict, clean_dict, parse_params
-import ckan.logic.action.get
+import ckan.lib.navl.dictization_functions as dict_fns
+import ckan.logic as logic
 import ckan.lib.search as search
-import ckan.new_authz
-
-from ckan.lib.plugins import lookup_group_plugin
+import ckan.model as model
+import ckan.new_authz as new_authz
+import ckan.lib.plugins
 import ckan.plugins as plugins
-
-try:
-    from collections import OrderedDict # 2.7
-except ImportError:
-    from sqlalchemy.util import OrderedDict
+from ckan.common import OrderedDict, c, g, request, _
 
 log = logging.getLogger(__name__)
 
+render = base.render
+abort = base.abort
 
-class GroupController(BaseController):
+NotFound = logic.NotFound
+NotAuthorized = logic.NotAuthorized
+ValidationError = logic.ValidationError
+check_access = logic.check_access
+get_action = logic.get_action
+tuplize_dict = logic.tuplize_dict
+clean_dict = logic.clean_dict
+parse_params = logic.parse_params
+
+lookup_group_plugin = ckan.lib.plugins.lookup_group_plugin
+
+
+class GroupController(base.BaseController):
 
     group_type = 'group'
 
@@ -47,7 +54,7 @@ class GroupController(BaseController):
 
     def _setup_template_variables(self, context, data_dict, group_type=None):
         return lookup_group_plugin(group_type).\
-        setup_template_variables(context, data_dict)
+            setup_template_variables(context, data_dict)
 
     def _new_template(self, group_type):
         return lookup_group_plugin(group_type).new_template()
@@ -73,6 +80,9 @@ class GroupController(BaseController):
     def _admins_template(self, group_type):
         return lookup_group_plugin(group_type).admins_template()
 
+    def _bulk_process_template(self, group_type):
+        return lookup_group_plugin(group_type).bulk_process_template()
+
     ## end hooks
     def _replace_group_org(self, string):
         ''' substitute organization for group if this is an org'''
@@ -128,8 +138,11 @@ class GroupController(BaseController):
                    'user': c.user or c.author, 'for_view': True,
                    'with_private': False}
 
-        data_dict = {'all_fields': True}
-
+        q = c.q = request.params.get('q', '')
+        data_dict = {'all_fields': True, 'q': q}
+        sort_by = c.sort_by_selected = request.params.get('sort')
+        if sort_by:
+            data_dict['sort'] = sort_by
         try:
             self._check_access('site_read', context)
         except NotAuthorized:
@@ -143,7 +156,7 @@ class GroupController(BaseController):
 
         results = self._action('group_list')(context, data_dict)
 
-        c.page = Page(
+        c.page = h.Page(
             collection=results,
             page=request.params.get('page', 1),
             url=h.pager_url,
@@ -151,14 +164,14 @@ class GroupController(BaseController):
         )
         return render(self._index_template(group_type))
 
-    def read(self, id):
-        from ckan.lib.search import SearchError
+    def read(self, id, limit=20):
         group_type = self._get_group_type(id.split('@')[0])
         context = {'model': model, 'session': model.Session,
                    'user': c.user or c.author,
                    'schema': self._db_to_form_schema(group_type=group_type),
                    'for_view': True}
         data_dict = {'id': id}
+
         # unicode format (decoded from utf8)
         q = c.q = request.params.get('q', '')
 
@@ -170,6 +183,18 @@ class GroupController(BaseController):
         except NotAuthorized:
             abort(401, _('Unauthorized to read group %s') % id)
 
+        self._read(id, limit)
+        return render(self._read_template(c.group_dict['type']))
+
+    def _read(self, id, limit):
+        ''' This is common code used by both read and bulk_process'''
+        group_type = self._get_group_type(id.split('@')[0])
+        context = {'model': model, 'session': model.Session,
+                   'user': c.user or c.author,
+                   'schema': self._db_to_form_schema(group_type=group_type),
+                   'for_view': True, 'extras_as_string': True}
+
+        q = c.q = request.params.get('q', '')
         # Search within group
         if c.group_dict.get('is_organization'):
             q += ' owner_org:"%s"' % c.group_dict.get('id')
@@ -182,9 +207,8 @@ class GroupController(BaseController):
 
         # c.group_admins is used by CKAN's legacy (Genshi) templates only,
         # if we drop support for those then we can delete this line.
-        c.group_admins = ckan.new_authz.get_group_or_org_admin_ids(c.group.id)
+        c.group_admins = new_authz.get_group_or_org_admin_ids(c.group.id)
 
-        limit = 20
         try:
             page = int(request.params.get('page', 1))
         except ValueError, e:
@@ -197,8 +221,17 @@ class GroupController(BaseController):
         sort_by = request.params.get('sort', None)
 
         def search_url(params):
-            url = self._url_for(controller='group', action='read',
-                            id=c.group_dict.get('name'))
+            if group_type == 'organization':
+                if c.action == 'bulk_process':
+                    url = self._url_for(controller='organization',
+                                        action='bulk_process',
+                                        id=id)
+                else:
+                    url = self._url_for(controller='organization',
+                                        action='read',
+                                        id=id)
+            else:
+                url = self._url_for(controller='group', action='read', id=id)
             params = [(k, v.encode('utf-8') if isinstance(v, basestring)
                        else str(v)) for k, v in params]
             return url + u'?' + urlencode(params)
@@ -213,8 +246,8 @@ class GroupController(BaseController):
 
         def remove_field(key, value=None, replace=None):
             return h.remove_url_param(key, value=value, replace=replace,
-                                  controller='group', action='read',
-                                  extras=dict(id=c.group_dict.get('name')))
+                                      controller='group', action='read',
+                                      extras=dict(id=c.group_dict.get('name')))
 
         c.remove_field = remove_field
 
@@ -246,9 +279,9 @@ class GroupController(BaseController):
             facets = OrderedDict()
 
             default_facet_titles = {'groups': _('Groups'),
-                              'tags': _('Tags'),
-                              'res_format': _('Formats'),
-                              'license': _('Licence'), }
+                                    'tags': _('Tags'),
+                                    'res_format': _('Formats'),
+                                    'license_id': _('License')}
 
             for facet in g.facets:
                 if facet in default_facet_titles:
@@ -265,7 +298,8 @@ class GroupController(BaseController):
                     facets = plugin.group_facets(
                         facets, self.group_type, None)
 
-            if 'capacity' in facets and (self.group_type != 'organization' or not user_member_of_orgs):
+            if 'capacity' in facets and (self.group_type != 'organization' or
+                                         not user_member_of_orgs):
                 del facets['capacity']
 
             c.facet_titles = facets
@@ -291,26 +325,83 @@ class GroupController(BaseController):
             )
 
             c.facets = query['facets']
-            maintain.deprecate_context_item(
-              'facets',
-              'Use `c.search_facets` instead.')
+            maintain.deprecate_context_item('facets',
+                                            'Use `c.search_facets` instead.')
 
             c.search_facets = query['search_facets']
             c.search_facets_limits = {}
             for facet in c.facets.keys():
-                limit = int(request.params.get('_%s_limit' % facet, 10))
+                limit = int(request.params.get('_%s_limit' % facet,
+                                               g.facets_default_number))
                 c.search_facets_limits[facet] = limit
             c.page.items = query['results']
 
             c.sort_by_selected = sort_by
 
-        except SearchError, se:
+        except search.SearchError, se:
             log.error('Group search error: %r', se.args)
             c.query_error = True
             c.facets = {}
             c.page = h.Page(collection=[])
 
-        return render(self._read_template(c.group_dict['type']))
+    def bulk_process(self, id):
+        ''' Allow bulk processing of datasets for an organization.  Make
+        private/public or delete. For organization admins.'''
+
+        group_type = self._get_group_type(id.split('@')[0])
+
+        if group_type != 'organization':
+            # FIXME: better error
+            raise Exception('Must be an organization')
+
+        # check we are org admin
+
+        context = {'model': model, 'session': model.Session,
+                   'user': c.user or c.author,
+                   'schema': self._db_to_form_schema(group_type=group_type),
+                   'for_view': True, 'extras_as_string': True}
+        data_dict = {'id': id}
+
+        try:
+            c.group_dict = self._action('group_show')(context, data_dict)
+            c.group = context['group']
+        except NotFound:
+            abort(404, _('Group not found'))
+        except NotAuthorized:
+            abort(401, _('Unauthorized to read group %s') % id)
+
+        # Search within group
+        action = request.params.get('bulk_action')
+        # If no action then just show the datasets
+        if not action:
+            # unicode format (decoded from utf8)
+            limit = 500
+            self._read(id, limit)
+            c.packages = c.page.items
+            return render(self._bulk_process_template(group_type))
+
+        # process the action first find the datasets to perform the action on.
+        # they are prefixed by dataset_ in the form data
+        datasets = []
+        for param in request.params:
+            if param.startswith('dataset_'):
+                datasets.append(param[8:])
+
+        action_functions = {
+            'private': 'bulk_update_private',
+            'public': 'bulk_update_public',
+            'delete': 'bulk_update_delete',
+        }
+
+        data_dict = {'datasets': datasets, 'org_id': c.group_dict['id']}
+
+        try:
+            get_action(action_functions[action])(context, data_dict)
+        except NotAuthorized:
+            abort(401, _('Not authorized to perform bulk update'))
+        base.redirect(h.url_for(controller='organization',
+                                action='bulk_process',
+                                id=id))
 
     def new(self, data=None, errors=None, error_summary=None):
         group_type = self._guess_group_type(True)
@@ -392,7 +483,7 @@ class GroupController(BaseController):
 
     def _save_new(self, context, group_type=None):
         try:
-            data_dict = clean_dict(unflatten(
+            data_dict = clean_dict(dict_fns.unflatten(
                 tuplize_dict(parse_params(request.params))))
             data_dict['type'] = group_type or 'group'
             context['message'] = data_dict.get('log_message', '')
@@ -405,7 +496,7 @@ class GroupController(BaseController):
             abort(401, _('Unauthorized to read group %s') % '')
         except NotFound, e:
             abort(404, _('Group not found'))
-        except DataError:
+        except dict_fns.DataError:
             abort(400, _(u'Integrity Error'))
         except ValidationError, e:
             errors = e.error_dict
@@ -423,7 +514,7 @@ class GroupController(BaseController):
 
     def _save_edit(self, id, context):
         try:
-            data_dict = clean_dict(unflatten(
+            data_dict = clean_dict(dict_fns.unflatten(
                 tuplize_dict(parse_params(request.params))))
             context['message'] = data_dict.get('log_message', '')
             data_dict['id'] = id
@@ -433,12 +524,12 @@ class GroupController(BaseController):
             if id != group['name']:
                 self._force_reindex(group)
 
-            h.redirect_to('%s_read' % str(group['type']), id=group['name'])
+            h.redirect_to('%s_read' % group['type'], id=group['name'])
         except NotAuthorized:
             abort(401, _('Unauthorized to read group %s') % id)
         except NotFound, e:
             abort(404, _('Group not found'))
-        except DataError:
+        except dict_fns.DataError:
             abort(400, _(u'Integrity Error'))
         except ValidationError, e:
             errors = e.error_dict
@@ -462,8 +553,8 @@ class GroupController(BaseController):
             c.authz_editable = False
         if not c.authz_editable:
             abort(401,
-                  gettext('User %r not authorized to edit %s authorizations') %
-                         (c.user, id))
+                  _('User %r not authorized to edit %s authorizations') %
+                   (c.user, id))
 
         roles = self._handle_update_of_authz(group)
         self._prepare_authz_info_for_render(roles)
@@ -501,8 +592,9 @@ class GroupController(BaseController):
                    'user': c.user or c.author}
 
         try:
-            c.members = self._action('member_list')(context, {'id': id,
-                                                              'object_type': 'user'})
+            c.members = self._action('member_list')(
+                context, {'id': id, 'object_type': 'user'}
+            )
             c.group_dict = self._action('group_show')(context, {'id': id})
         except NotAuthorized:
             abort(401, _('Unauthorized to delete group %s') % '')
@@ -517,7 +609,7 @@ class GroupController(BaseController):
         #self._check_access('group_delete', context, {'id': id})
         try:
             if request.method == 'POST':
-                data_dict = clean_dict(unflatten(
+                data_dict = clean_dict(dict_fns.unflatten(
                     tuplize_dict(parse_params(request.params))))
                 data_dict['id'] = id
                 c.group_dict = self._action('group_member_create')(context, data_dict)
@@ -526,7 +618,7 @@ class GroupController(BaseController):
                 user = request.params.get('user')
                 if user:
                     c.user_dict = get_action('user_show')(context, {'id': user})
-                    c.user_role = ckan.new_authz.users_role_for_group_or_org(id, user) or 'member'
+                    c.user_role = new_authz.users_role_for_group_or_org(id, user) or 'member'
                 else:
                     c.user_role = 'member'
                 c.group_dict = self._action('group_show')(context, {'id': id})
@@ -535,6 +627,8 @@ class GroupController(BaseController):
             abort(401, _('Unauthorized to add member to group %s') % '')
         except NotFound:
             abort(404, _('Group not found'))
+        except ValidationError, e:
+            h.flash_error(e.error_summary)
         return self._render_template('group/member_new.html')
 
     def member_delete(self, id):
@@ -564,7 +658,6 @@ class GroupController(BaseController):
             abort(404, _('Group not found'))
         return self._render_template('group/confirm_delete_member.html')
 
-
     def history(self, id):
         if 'diff' in request.params or 'selected1' in request.params:
             try:
@@ -588,7 +681,7 @@ class GroupController(BaseController):
         try:
             c.group_dict = self._action('group_show')(context, data_dict)
             c.group_revisions = self._action('group_revision_list')(context,
-                                                                  data_dict)
+                                                                    data_dict)
             #TODO: remove
             # Still necessary for the authz check in group/layout.html
             c.group = context['group']
@@ -604,7 +697,7 @@ class GroupController(BaseController):
             feed = Atom1Feed(
                 title=_(u'CKAN Group Revision History'),
                 link=self._url_for(controller='group', action='read',
-                               id=c.group_dict['name']),
+                                   id=c.group_dict['name']),
                 description=_(u'Recent changes to CKAN Group: ') +
                 c.group_dict['display_name'],
                 language=unicode(get_lang()),
@@ -661,7 +754,7 @@ class GroupController(BaseController):
         # Add the group's activity stream (already rendered to HTML) to the
         # template context for the group/read.html template to retrieve later.
         c.group_activity_stream = get_action('group_activity_list_html')(
-                context, {'id': c.group_dict['id'], 'offset': offset})
+            context, {'id': c.group_dict['id'], 'offset': offset})
 
         #return render('group/activity_stream.html')
         return render(self._activity_template(c.group_dict['type']))
@@ -679,7 +772,7 @@ class GroupController(BaseController):
                 group_dict['title']))
         except ValidationError as e:
             error_message = (e.extra_msg or e.message or e.error_summary
-                    or e.error_dict)
+                             or e.error_dict)
             h.flash_error(error_message)
         except NotAuthorized as e:
             h.flash_error(e.extra_msg)
@@ -698,7 +791,7 @@ class GroupController(BaseController):
                 group_dict['title']))
         except ValidationError as e:
             error_message = (e.extra_msg or e.message or e.error_summary
-                    or e.error_dict)
+                             or e.error_dict)
             h.flash_error(error_message)
         except (NotFound, NotAuthorized) as e:
             error_message = e.extra_msg or e.message
@@ -710,15 +803,14 @@ class GroupController(BaseController):
                    'user': c.user or c.author}
         c.group_dict = self._get_group_dict(id)
         try:
-            c.followers = get_action('group_follower_list')(context,
-                    {'id': id})
+            c.followers = get_action('group_follower_list')(context, {'id': id})
         except NotAuthorized:
             abort(401, _('Unauthorized to view followers %s') % '')
         return render('group/followers.html')
 
     def admins(self, id):
         c.group_dict = self._get_group_dict(id)
-        c.admins = ckan.new_authz.get_group_or_org_admin_ids(id)
+        c.admins = new_authz.get_group_or_org_admin_ids(id)
         return render(self._admins_template(c.group_dict['type']))
 
     def about(self, id):
@@ -751,7 +843,7 @@ class GroupController(BaseController):
         validation = fs.validate()
         if not validation:
             c.form = self._render_edit_form(fs)
-            raise ValidationException(fs)
+            raise base.ValidationException(fs)
 
         try:
             fs.sync()
@@ -765,7 +857,7 @@ class GroupController(BaseController):
         validation = fs.validate()
         if not validation:
             c.form = self._render_edit_form(fs)
-            raise ValidationException(fs)
+            raise base.ValidationException(fs)
         try:
             fs.sync()
         except Exception, inst:
