@@ -1,17 +1,16 @@
+import urllib
+
 import ckan.model as model
 import ckan.logic as logic
 import ckan.lib.base as base
 import ckan.lib.helpers as h
 import ckan.lib.navl.dictization_functions as df
 
-import pylons.i18n as i18n
+from ckan.common import _, c
 
-_ = i18n._
-import urllib
 
-c = base.c
 abort = base.abort
-_get_action=logic.get_action
+_get_action = logic.get_action
 
 
 class RelatedController(base.BaseController):
@@ -32,15 +31,15 @@ class RelatedController(base.BaseController):
             'featured': base.request.params.get('featured', '')
         }
 
-        params_nopage = [(k, v) for k,v in base.request.params.items()
+        params_nopage = [(k, v) for k, v in base.request.params.items()
                          if k != 'page']
         try:
             page = int(base.request.params.get('page', 1))
-        except ValueError, e:
+        except ValueError:
             base.abort(400, ('"page" parameter must be an integer'))
 
         # Update ordering in the context
-        query = logic.get_action('related_list')(context,data_dict)
+        query = logic.get_action('related_list')(context, data_dict)
 
         def search_url(params):
             url = h.url_for(controller='related', action='dashboard')
@@ -54,7 +53,6 @@ class RelatedController(base.BaseController):
             params.append(('page', page))
             return search_url(params)
 
-
         c.page = h.Page(
             collection=query.all(),
             page=page,
@@ -66,13 +64,15 @@ class RelatedController(base.BaseController):
         c.filters = dict(params_nopage)
 
         c.type_options = self._type_options()
-        c.sort_options = ({'value': '', 'text': _('Most viewed')},
-                          {'value': 'view_count_desc', 'text': _('Most Viewed')},
-                          {'value': 'view_count_asc', 'text': _('Least Viewed')},
-                          {'value': 'created_desc', 'text': _('Newest')},
-                          {'value': 'created_asc', 'text': _('Oldest')})
+        c.sort_options = (
+            {'value': '', 'text': _('Most viewed')},
+            {'value': 'view_count_desc', 'text': _('Most Viewed')},
+            {'value': 'view_count_asc', 'text': _('Least Viewed')},
+            {'value': 'created_desc', 'text': _('Newest')},
+            {'value': 'created_asc', 'text': _('Oldest')}
+        )
 
-        return base.render( "related/dashboard.html")
+        return base.render("related/dashboard.html")
 
     def read(self, id):
         context = {'model': model, 'session': model.Session,
@@ -85,8 +85,8 @@ class RelatedController(base.BaseController):
         except logic.NotAuthorized:
             base.abort(401, _('Not authorized to see this page'))
 
-        related = model.Session.query(model.Related).\
-                    filter(model.Related.id == id).first()
+        related = model.Session.query(model.Related) \
+            .filter(model.Related.id == id).first()
         if not related:
             base.abort(404, _('The requested related item was not found'))
 
@@ -97,7 +97,6 @@ class RelatedController(base.BaseController):
 
         base.redirect(related.url)
 
-
     def list(self, id):
         """ List all related items for a specific dataset """
         context = {'model': model, 'session': model.Session,
@@ -164,10 +163,9 @@ class RelatedController(base.BaseController):
         if base.request.method == "POST":
             try:
                 data = logic.clean_dict(
-                        df.unflatten(
-                            logic.tuplize_dict(
-                                logic.parse_params(base.request.params)
-                        )))
+                    df.unflatten(
+                        logic.tuplize_dict(
+                            logic.parse_params(base.request.params))))
 
                 if is_edit:
                     data['id'] = related_id
@@ -182,9 +180,8 @@ class RelatedController(base.BaseController):
                 else:
                     h.flash_success(_("Related item was successfully updated"))
 
-                h.redirect_to(controller='related',
-                               action='list',
-                               id=c.pkg_dict['name'])
+                h.redirect_to(
+                    controller='related', action='list', id=c.pkg_dict['name'])
             except df.DataError:
                 base.abort(400, _(u'Integrity Error'))
             except logic.ValidationError, e:
@@ -202,7 +199,6 @@ class RelatedController(base.BaseController):
         return base.render(tpl)
 
     def delete(self, id, related_id):
-
         if 'cancel' in base.request.params:
             h.redirect_to(controller='related', action='edit',
                           id=id, related_id=related_id)
@@ -215,7 +211,8 @@ class RelatedController(base.BaseController):
                 logic.get_action('related_delete')(context, {'id': related_id})
                 h.flash_notice(_('Related item has been deleted.'))
                 h.redirect_to(controller='package', action='read', id=id)
-            c.related_dict = logic.get_action('related_show')(context, {'id': related_id})
+            c.related_dict = logic.get_action('related_show')(
+                context, {'id': related_id})
             c.pkg_id = id
         except logic.NotAuthorized:
             base.abort(401, _('Unauthorized to delete related item %s') % '')
