@@ -1,10 +1,9 @@
 import logging
 from urllib import quote
+from urlparse import urlparse
 
-from pylons import session, c, g, request, config
-from pylons.i18n import _
+from pylons import config
 
-import ckan.lib.i18n as i18n
 import ckan.lib.base as base
 import ckan.model as model
 import ckan.lib.helpers as h
@@ -14,6 +13,9 @@ import ckan.logic.schema as schema
 import ckan.lib.captcha as captcha
 import ckan.lib.mailer as mailer
 import ckan.lib.navl.dictization_functions as dictization_functions
+import ckan.plugins as p
+
+from ckan.common import _, c, g, request
 
 log = logging.getLogger(__name__)
 
@@ -33,7 +35,6 @@ unflatten = dictization_functions.unflatten
 
 
 class UserController(base.BaseController):
-
     def __before__(self, action, **env):
         base.BaseController.__before__(self, action, **env)
         try:
@@ -139,6 +140,12 @@ class UserController(base.BaseController):
                       id=user_ref)
 
     def register(self, data=None, errors=None, error_summary=None):
+        context = {'model': model, 'session': model.Session, 'user': c.user}
+        try:
+            check_access('user_create', context)
+        except NotAuthorized:
+            abort(401, _('Unauthorized to register as a user.'))
+
         return self.new(data, errors, error_summary)
 
     def new(self, data=None, errors=None, error_summary=None):
@@ -211,6 +218,8 @@ class UserController(base.BaseController):
     def edit(self, id=None, data=None, errors=None, error_summary=None):
         context = {'save': 'save' in request.params,
                    'schema': self._edit_form_to_db_schema(),
+                   'model': model, 'session': model.Session,
+                   'user': c.user,
                    }
         if id is None:
             if c.userobj:
@@ -219,6 +228,11 @@ class UserController(base.BaseController):
                 abort(400, _('No user specified'))
         data_dict = {'id': id}
 
+        try:
+            check_access('user_update', context, data_dict)
+        except NotAuthorized:
+            abort(401, _('Unauthorized to edit a user.'))
+
         if (context['save']) and not data:
             return self._save_edit(id, context)
 
@@ -236,7 +250,7 @@ class UserController(base.BaseController):
 
         except NotAuthorized:
             abort(401, _('Unauthorized to edit user %s') % '')
-        except NotFound, e:
+        except NotFound:
             abort(404, _('User not found'))
 
         user_obj = context.get('user_obj')
@@ -287,11 +301,10 @@ class UserController(base.BaseController):
             return self.edit(id, data_dict, errors, error_summary)
 
     def login(self, error=None):
-        lang = session.pop('lang', None)
-        if lang:
-            session.save()
-            return h.redirect_to(locale=str(lang), controller='user',
-                                 action='login')
+        # Do any plugin login stuff
+        for item in p.PluginImplementations(p.IAuthenticator):
+            item.login()
+
         if 'error' in request.params:
             h.flash_error(request.params['error'])
 
@@ -301,7 +314,10 @@ class UserController(base.BaseController):
             g.openid_enabled = False
 
         if not c.user:
-            came_from = request.params.get('came_from', '')
+            came_from = request.params.get('came_from')
+            if not came_from:
+                came_from = h.url_for(controller='user', action='logged_in',
+                                      __ckan_no_root=True)
             c.login_handler = h.url_for(
                 self._get_repoze_handler('login_handler_path'),
                 came_from=came_from)
@@ -314,14 +330,10 @@ class UserController(base.BaseController):
             return render('user/logout_first.html')
 
     def logged_in(self):
-        # we need to set the language via a redirect
-        lang = session.pop('lang', None)
-        session.save()
+        # redirect if needed
         came_from = request.params.get('came_from', '')
-
-        # we need to set the language explicitly here or the flash
-        # messages will not be translated.
-        i18n.set_lang(lang)
+        if self._sane_came_from(came_from):
+            return h.redirect_to(str(came_from))
 
         if c.user:
             context = None
@@ -331,8 +343,6 @@ class UserController(base.BaseController):
 
             h.flash_success(_("%s is now logged in") %
                             user_dict['display_name'])
-            if came_from:
-                return h.redirect_to(str(came_from))
             return self.me()
         else:
             err = _('Login failed. Bad username or password.')
@@ -341,34 +351,38 @@ class UserController(base.BaseController):
                          'with a user account.)')
             if h.asbool(config.get('ckan.legacy_templates', 'false')):
                 h.flash_error(err)
-                h.redirect_to(locale=lang, controller='user',
+                h.redirect_to(controller='user',
                               action='login', came_from=came_from)
             else:
                 return self.login(error=err)
 
     def logout(self):
-        # save our language in the session so we don't lose it
-        session['lang'] = request.environ.get('CKAN_LANG')
-        session.save()
-        h.redirect_to(self._get_repoze_handler('logout_handler_path'))
-
-    def set_lang(self, lang):
-        # this allows us to set the lang in session.  Used for logging
-        # in/out to prevent being lost when repoze.who redirects things
-        session['lang'] = str(lang)
-        session.save()
+        # Do any plugin logout stuff
+        for item in p.PluginImplementations(p.IAuthenticator):
+            item.logout()
+        url = h.url_for(controller='user', action='logged_out_page',
+                        __ckan_no_root=True)
+        h.redirect_to(self._get_repoze_handler('logout_handler_path') +
+                      '?came_from=' + url)
 
     def logged_out(self):
-        # we need to get our language info back and the show the correct page
-        lang = session.get('lang')
-        c.user = None
-        session.delete()
-        h.redirect_to(locale=lang, controller='user', action='logged_out_page')
+        # redirect if needed
+        came_from = request.params.get('came_from', '')
+        if self._sane_came_from(came_from):
+            return h.redirect_to(str(came_from))
+        h.redirect_to(controller='user', action='logged_out_page')
 
     def logged_out_page(self):
         return render('user/logout.html')
 
     def request_reset(self):
+        context = {'model': model, 'session': model.Session, 'user': c.user}
+        data_dict = {'id': request.params.get('user')}
+        try:
+            check_access('request_reset', context)
+        except NotAuthorized:
+            abort(401, _('Unauthorized to request reset password.'))
+
         if request.method == 'POST':
             id = request.params.get('user')
 
@@ -423,6 +437,11 @@ class UserController(base.BaseController):
 
         data_dict = {'id': id}
 
+        try:
+            check_access('user_reset', context)
+        except NotAuthorized:
+            abort(401, _('Unauthorized to reset password.'))
+
         try:
             user_dict = get_action('user_show')(context, data_dict)
 
@@ -505,8 +524,7 @@ class UserController(base.BaseController):
 
         return render('user/activity_stream.html')
 
-    def _get_dashboard_context(self, filter_type=None, filter_id=None,
-            q=None):
+    def _get_dashboard_context(self, filter_type=None, filter_id=None, q=None):
         '''Return a dict needed by the dashboard view to determine context.'''
 
         def display_name(followee):
@@ -518,8 +536,10 @@ class UserController(base.BaseController):
             return display_name or fullname or title or name
 
         if (filter_type and filter_id):
-            context = {'model': model, 'session': model.Session,
-                   'user': c.user or c.author, 'for_view': True}
+            context = {
+                'model': model, 'session': model.Session,
+                'user': c.user or c.author, 'for_view': True
+            }
             data_dict = {'id': filter_id}
             followee = None
 
@@ -527,11 +547,12 @@ class UserController(base.BaseController):
                 'dataset': 'package_show',
                 'user': 'user_show',
                 'group': 'group_show'
-                }
-            action_function = logic.get_action(action_functions.get(filter_type))
+            }
+            action_function = logic.get_action(
+                action_functions.get(filter_type))
             # Is this a valid type?
             if action_function is None:
-                raise abort(404, _('Follow item not found'))
+                abort(404, _('Follow item not found'))
             try:
                 followee = action_function(context, data_dict)
             except NotFound:
@@ -581,6 +602,24 @@ class UserController(base.BaseController):
 
         return render('user/dashboard.html')
 
+    def dashboard_datasets(self):
+        context = {'for_view': True}
+        data_dict = {'user_obj': c.userobj}
+        self._setup_template_variables(context, data_dict)
+        return render('user/dashboard_datasets.html')
+
+    def dashboard_organizations(self):
+        context = {'for_view': True}
+        data_dict = {'user_obj': c.userobj}
+        self._setup_template_variables(context, data_dict)
+        return render('user/dashboard_organizations.html')
+
+    def dashboard_groups(self):
+        context = {'for_view': True}
+        data_dict = {'user_obj': c.userobj}
+        self._setup_template_variables(context, data_dict)
+        return render('user/dashboard_groups.html')
+
     def follow(self, id):
         '''Start following this user.'''
         context = {'model': model,
@@ -619,3 +658,14 @@ class UserController(base.BaseController):
                              or e.error_dict)
             h.flash_error(error_message)
         h.redirect_to(controller='user', action='read', id=id)
+
+    def _sane_came_from(self, url):
+        '''Returns True if came_from is local'''
+        if not url or (len(url) >= 2 and url.startswith('//')):
+            return False
+        parsed = urlparse(url)
+        if parsed.scheme:
+            domain = urlparse(h.url_for('/', qualified=True)).netloc
+            if domain != parsed.netloc:
+                return False
+        return True
