@@ -17,7 +17,8 @@ import domain_object
 import activity
 import extension
 
-import ckan.lib.dictization
+import ckan.lib.maintain as maintain
+import ckan.lib.dictization as dictization
 
 __all__ = ['Package', 'package_table', 'package_revision_table',
            'PACKAGE_NAME_MAX_LENGTH', 'PACKAGE_NAME_MIN_LENGTH',
@@ -510,11 +511,15 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
             return datetime.datetime.utcfromtimestamp(timestamp_float)
 
     @property
+    @maintain.deprecated('`is_private` attriute of model.Package is ' +
+                         'deprecated and should not be used.  Use `private`')
     def is_private(self):
         """
+        DEPRECATED in 2.1
+
         A package is private if belongs to any private groups
         """
-        return bool(self.get_groups(capacity='private'))
+        return self.private
 
     def is_in_group(self, group):
         return group in self.get_groups()
@@ -590,7 +595,7 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
                 activity_type = 'deleted'
 
         try:
-            d = {'package': ckan.lib.dictization.table_dictize(self,
+            d = {'package': dictization.table_dictize(self,
                 context={'model': ckan.model})}
             return activity.Activity(user_id, self.id, revision.id,
                     "%s package" % activity_type, d)
@@ -611,7 +616,7 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
         if activity_type == 'changed' and self.state == u'deleted':
             activity_type = 'deleted'
 
-        package_dict = ckan.lib.dictization.table_dictize(self,
+        package_dict = dictization.table_dictize(self,
                 context={'model':ckan.model})
         return activity.ActivityDetail(activity_id, self.id, u"Package", activity_type,
             {'package': package_dict })
