@@ -1,17 +1,13 @@
 import sys
+import re
 from logging import getLogger
 
-try:
-    from collections import OrderedDict # 2.7
-except ImportError:
-    from sqlalchemy.util import OrderedDict
-
-from pylons import config, c
-from pylons.i18n import _
+from pylons import config
 from paste.deploy.converters import asbool
 
 import ckan.plugins as p
 import ckan.model as model
+from ckan.common import OrderedDict, _, c
 
 log = getLogger(__name__)
 
@@ -23,6 +19,14 @@ class AuthFunctions:
 def clear_auth_functions_cache():
     AuthFunctions._functions.clear()
 
+
+def clean_action_name(action_name):
+    ''' Used to convert old style action names into new style ones '''
+    new_action_name = re.sub('package', 'dataset', action_name)
+    # CS: bad_spelling ignore
+    return re.sub('licence', 'license', new_action_name)
+
+
 def is_sysadmin(username):
     ''' returns True is username is a sysadmin '''
     if not username:
@@ -63,12 +67,16 @@ def is_authorized(action, context, data_dict=None):
     if context.get('ignore_auth'):
         return {'success': True}
 
-    # sysadmins can do anything
-    if is_sysadmin(context.get('user')):
-        return {'success': True}
-
+    action = clean_action_name(action)
     auth_function = _get_auth_function(action)
     if auth_function:
+        # sysadmins can do anything unless the auth_sysadmins_check
+        # decorator was used in which case they are treated like all other
+        # users.
+        if is_sysadmin(context.get('user')):
+            if not getattr(auth_function, 'auth_sysadmins_check', False):
+                return {'success': True}
+
         return auth_function(context, data_dict)
     else:
         raise ValueError(_('Authorization function not found: %s' % action))
@@ -213,8 +221,7 @@ def get_user_id_for_username(user_name, allow_none=False):
         return None
     raise Exception('Not logged in user')
 
-def _get_auth_function(action, profile=None):
-    from pylons import config
+def _get_auth_function(action):
 
     if action in AuthFunctions._functions:
         return AuthFunctions._functions.get(action)
@@ -225,17 +232,7 @@ def _get_auth_function(action, profile=None):
     # to load anything from ckan.auth that looks like it might
     # be an authorisation function
 
-    # We will load the auth profile from settings
     module_root = 'ckan.logic.auth'
-    if profile is not None:
-        auth_profile = profile
-    else:
-        auth_profile = config.get('ckan.auth.profile', '')
-
-    if auth_profile:
-        module_root = '%s.%s' % (module_root, auth_profile)
-
-    log.debug('Using auth profile at %s' % module_root)
 
     for auth_module_name in ['get', 'create', 'update','delete']:
         module_path = '%s.%s' % (module_root, auth_module_name,)
@@ -250,6 +247,7 @@ def _get_auth_function(action, profile=None):
 
         for key, v in module.__dict__.items():
             if not key.startswith('_'):
+                key = clean_action_name(key)
                 AuthFunctions._functions[key] = v
 
     # Then overwrite them with any specific ones in the plugins:
@@ -257,6 +255,7 @@ def _get_auth_function(action, profile=None):
     fetched_auth_functions = {}
     for plugin in p.PluginImplementations(p.IAuthFunctions):
         for name, auth_function in plugin.get_auth_functions().items():
+            name = clean_action_name(name)
             if name in resolved_auth_function_plugins:
                 raise Exception(
                     'The auth function %r is already implemented in %r' % (
