@@ -10,6 +10,7 @@ import ckan
 from ckan.lib.create_test_data import CreateTestData
 from ckan.lib.dictization.model_dictize import resource_dictize
 import ckan.model as model
+import ckan.tests as tests
 from ckan.tests import WsgiAppCase
 from ckan.tests.functional.api import assert_dicts_equal_ignoring_ordering
 from ckan.tests import setup_test_search_index, search_related
@@ -67,12 +68,54 @@ class TestAction(WsgiAppCase):
         assert res['help'].startswith(
             "Return a list of the names of the site's datasets (packages).")
 
-        # Test GET request
+		# Test GET request
         res = json.loads(self.app.get('/api/action/package_list').body)
         assert len(res['result']) == 2
         assert 'warandpeace' in res['result']
         assert 'annakarenina' in res['result']
 
+    def test_01_current_package_list_with_resources(self):
+        url = '/api/action/current_package_list_with_resources'
+
+        postparams = '%s=1' % json.dumps({
+            'limit': 1,
+            'offset': 1})
+        res = json.loads(self.app.post(url, params=postparams).body)
+        assert res['success']
+        assert len(res['result']) == 1
+
+        postparams = '%s=1' % json.dumps({
+            'limit': '5'})
+        res = json.loads(self.app.post(url, params=postparams).body)
+        assert res['success']
+
+        postparams = '%s=1' % json.dumps({
+            'limit': -2})
+        res = json.loads(self.app.post(url, params=postparams,
+                         status=StatusCodes.STATUS_409_CONFLICT).body)
+        assert not res['success']
+
+        postparams = '%s=1' % json.dumps({
+            'offset': 'a'})
+        res = json.loads(self.app.post(url, params=postparams,
+                         status=StatusCodes.STATUS_409_CONFLICT).body)
+        assert not res['success']
+
+        postparams = '%s=1' % json.dumps({
+            'limit': 2,
+            'page': 1})
+        res = json.loads(self.app.post(url, params=postparams).body)
+        assert res['success']
+        assert len(res['result']) == 2
+
+        postparams = '%s=1' % json.dumps({
+            'limit': 1,
+            'page': 0})
+        res = json.loads(self.app.post(url,
+                         params=postparams,
+                         status=StatusCodes.STATUS_409_CONFLICT).body)
+        assert not res['success']
+
     def test_01_package_show(self):
         anna_id = model.Package.by_name(u'annakarenina').id
         postparams = '%s=1' % json.dumps({'id': anna_id})
@@ -104,7 +147,7 @@ class TestAction(WsgiAppCase):
         assert not missing_keys, missing_keys
 
     def test_02_package_autocomplete_match_name(self):
-        postparams = '%s=1' % json.dumps({'q':'war'})
+        postparams = '%s=1' % json.dumps({'q':'war', 'limit': 5})
         res = self.app.post('/api/action/package_autocomplete', params=postparams)
         res_obj = json.loads(res.body)
         assert_equal(res_obj['success'], True)
@@ -115,7 +158,7 @@ class TestAction(WsgiAppCase):
         assert_equal(res_obj['result'][0]['match_displayed'], 'warandpeace')
 
     def test_02_package_autocomplete_match_title(self):
-        postparams = '%s=1' % json.dumps({'q':'a%20w'})
+        postparams = '%s=1' % json.dumps({'q':'a%20w', 'limit': 5})
         res = self.app.post('/api/action/package_autocomplete', params=postparams)
         res_obj = json.loads(res.body)
         assert_equal(res_obj['success'], True)
@@ -181,12 +224,10 @@ class TestAction(WsgiAppCase):
 
     def test_03_create_private_package(self):
 
-        def _do_request(package_dict):
-            postparams = '%s=1' % json.dumps(package_dict)
-            res = self.app.post('/api/action/package_create', params=postparams,
-                            extra_environ={'Authorization': str(self.sysadmin_user.apikey)})
-            package_created = json.loads(res.body)['result']
-            return package_created
+        # Make an organization, because private datasets must belong to one.
+        organization = tests.call_action_api(self.app, 'organization_create',
+                                             name='test_org',
+                                             apikey=self.sysadmin_user.apikey)
 
         # Create a dataset without specifying visibility
         package_dict = {
@@ -212,24 +253,32 @@ class TestAction(WsgiAppCase):
             'tags': [{'name': u'russian'}, {'name': u'tolstoy'}],
             'title': u'A Novel By Tolstoy',
             'url': u'http://www.annakarenina.com',
+            'owner_org': organization['id'],
             'version': u'0.7a',
         }
-
-        package_created = _do_request(package_dict)
+        package_created = tests.call_action_api(self.app, 'package_create',
+                                              apikey=self.sysadmin_user.apikey,
+                                              **package_dict)
         assert package_created['private'] is False
 
         # Create a new one, explicitly saying it is public
         package_dict['name'] = u'annakareninanew_vis_public'
         package_dict['private'] = False
 
-        package_created_public = _do_request(package_dict)
+        package_created_public = tests.call_action_api(self.app,
+                                              'package_create',
+                                              apikey=self.sysadmin_user.apikey,
+                                              **package_dict)
         assert package_created_public['private'] is False
 
         # Create a new one, explicitly saying it is private
         package_dict['name'] = u'annakareninanew_vis_private'
         package_dict['private'] = True
 
-        package_created_private = _do_request(package_dict)
+        package_created_private = tests.call_action_api(self.app,
+                                              'package_create',
+                                              apikey=self.sysadmin_user.apikey,
+                                              **package_dict)
         assert package_created_private['private'] is True
 
 
@@ -585,12 +634,14 @@ class TestAction(WsgiAppCase):
     def test_16_user_autocomplete(self):
         #Empty query
         postparams = '%s=1' % json.dumps({})
-        res = self.app.post('/api/action/user_autocomplete', params=postparams)
+        res = self.app.post(
+            '/api/action/user_autocomplete',
+            params=postparams,
+            status=StatusCodes.STATUS_409_CONFLICT)
         res_obj = json.loads(res.body)
         assert res_obj['help'].startswith(
                 "Return a list of user names that contain a string.")
-        assert res_obj['result'] == []
-        assert res_obj['success'] is True
+        assert res_obj['success'] is False
 
         #Normal query
         postparams = '%s=1' % json.dumps({'q':'joe'})
@@ -852,7 +903,7 @@ class TestAction(WsgiAppCase):
         assert group_names == set(['annakarenina', 'warandpeace']), group_names
 
     def test_29_group_package_show_pending(self):
-        context = {'model': model, 'session': model.Session, 'user': self.sysadmin_user.name, 'api_version': 2}
+        context = {'model': model, 'session': model.Session, 'user': self.sysadmin_user.name, 'api_version': 2, 'ignore_auth': True}
         group = {
             'name': 'test_group_pending_package',
             'packages': [{'id': model.Package.get('annakarenina').id}]
@@ -888,7 +939,7 @@ class TestAction(WsgiAppCase):
         postparams = '%s=1' % json.dumps('not a dict')
         res = self.app.post('/api/action/package_list', params=postparams,
                             status=400)
-        assert 'Request data JSON decoded to u\'not a dict\' but it needs to be a dictionary.' in res.body, res.body
+        assert "Bad request - JSON Error: Request data JSON decoded to 'not a dict' but it needs to be a dictionary." in res.body, res.body
 
     def test_31_bad_request_format_not_json(self):
         postparams = '=1'
@@ -1278,7 +1329,7 @@ class TestActionPackageSearch(WsgiAppCase):
     def test_1_basic(self):
         params = {
                 'q':'tolstoy',
-                'facet.field': ('groups', 'tags', 'res_format', 'license'),
+                'facet.field': ['groups', 'tags', 'res_format', 'license'],
                 'rows': 20,
                 'start': 0,
             }
@@ -1291,7 +1342,9 @@ class TestActionPackageSearch(WsgiAppCase):
         assert_equal(result['results'][0]['name'], 'annakarenina')
 
         # Test GET request
-        url_params = urllib.urlencode(params)
+        params_json_list = params
+        params_json_list['facet.field'] = json.dumps(params['facet.field'])
+        url_params = urllib.urlencode(params_json_list)
         res = self.app.get('/api/action/package_search?{0}'.format(url_params))
         res = json.loads(res.body)
         result = res['result']
@@ -1335,7 +1388,7 @@ class TestActionPackageSearch(WsgiAppCase):
         res = self.app.post('/api/action/package_search', params=postparams,
                             status=400)
         assert '"message": "Search Query is invalid:' in res.body, res.body
-        assert '"Invalid search parameters: [u\'weird_param\']' in res.body, res.body
+        assert '"Invalid search parameters: [\'weird_param\']' in res.body, res.body
 
     def test_4_sort_by_metadata_modified(self):
         search_params = '%s=1' % json.dumps({
@@ -1504,3 +1557,86 @@ class TestSearchPluginInterface(WsgiAppCase):
 
         res = self.app.get('/dataset?q=')
         assert res.body.count('string_not_found_in_rest_of_template') == 2
+
+
+class TestBulkActions(WsgiAppCase):
+
+    @classmethod
+    def setup_class(cls):
+        search.clear()
+        model.Session.add_all([
+            model.User(name=u'sysadmin', apikey=u'sysadmin',
+                       password=u'sysadmin', sysadmin=True),
+        ])
+        model.Session.commit()
+
+        data_dict = '%s=1' % json.dumps({
+            'name': 'org',
+        })
+        res = cls.app.post('/api/action/organization_create',
+                            extra_environ={'Authorization': 'sysadmin'},
+                            params=data_dict)
+        cls.org_id = json.loads(res.body)['result']['id']
+
+        cls.package_ids = []
+        for i in range(0,12):
+            data_dict = '%s=1' % json.dumps({
+                'name': 'name{i}'.format(i=i),
+                'owner_org': 'org',
+            })
+            res = cls.app.post('/api/action/package_create',
+                                extra_environ={'Authorization': 'sysadmin'},
+                                params=data_dict)
+            cls.package_ids.append(json.loads(res.body)['result']['id'])
+
+
+    @classmethod
+    def teardown_class(self):
+        model.repo.rebuild_db()
+
+    def test_01_make_private_then_public(self):
+        data_dict = '%s=1' % json.dumps({
+            'datasets': self.package_ids,
+            'org_id': self.org_id,
+        })
+        res = self.app.post('/api/action/bulk_update_private',
+                            extra_environ={'Authorization': 'sysadmin'},
+                            params=data_dict)
+
+        dataset_list = [row.private for row in
+                        model.Session.query(model.Package.private).all()]
+        assert len(dataset_list) == 12, len(dataset_list)
+        assert all(dataset_list)
+
+        res = self.app.get('/api/action/package_search?q=*:*')
+        assert json.loads(res.body)['result']['count'] == 0
+
+        res = self.app.post('/api/action/bulk_update_public',
+                            extra_environ={'Authorization': 'sysadmin'},
+                            params=data_dict)
+
+        dataset_list = [row.private for row in
+                        model.Session.query(model.Package.private).all()]
+        assert len(dataset_list) == 12, len(dataset_list)
+        assert not any(dataset_list)
+
+        res = self.app.get('/api/action/package_search?q=*:*')
+        assert json.loads(res.body)['result']['count'] == 12
+
+    def test_02_bulk_delete(self):
+
+        data_dict = '%s=1' % json.dumps({
+            'datasets': self.package_ids,
+            'org_id': self.org_id,
+        })
+        res = self.app.post('/api/action/bulk_update_delete',
+                            extra_environ={'Authorization': 'sysadmin'},
+                            params=data_dict)
+
+        dataset_list = [row.state for row in
+                        model.Session.query(model.Package.state).all()]
+        assert len(dataset_list) == 12, len(dataset_list)
+        assert all(state == 'deleted' for state in dataset_list)
+
+        res = self.app.get('/api/action/package_search?q=*:*')
+        assert json.loads(res.body)['result']['count'] == 0
