@@ -1,5 +1,4 @@
 import logging
-import pylons
 
 import ckan.plugins as p
 import ckanext.datastore.logic.action as action
@@ -20,6 +19,8 @@ class DatastorePlugin(p.SingletonPlugin):
     p.implements(p.IConfigurable, inherit=True)
     p.implements(p.IActions)
     p.implements(p.IAuthFunctions)
+    p.implements(p.IDomainObjectModification, inherit=True)
+    p.implements(p.IRoutes, inherit=True)
 
     legacy_mode = False
 
@@ -52,7 +53,9 @@ class DatastorePlugin(p.SingletonPlugin):
         else:
             self.read_url = self.config['ckan.datastore.read_url']
 
-        if not model.engine_is_pg():
+        read_engine = db._get_engine(
+            {'connection_url': self.read_url})
+        if not model.engine_is_pg(read_engine):
             log.warn('We detected that you do not use a PostgreSQL '
                      'database. The DataStore will NOT work and DataStore '
                      'tests will be skipped.')
@@ -69,19 +72,16 @@ class DatastorePlugin(p.SingletonPlugin):
 
         ## Do light wrapping around action function to add datastore_active
         ## to resource dict.  Not using IAction extension as this prevents
-        ## other plugins from having a custom resource_read.
+        ## other plugins from having a custom resource_show.
 
         # Make sure actions are cached
         resource_show = p.toolkit.get_action('resource_show')
 
+        @logic.side_effect_free
         def new_resource_show(context, data_dict):
-            engine = db._get_engine(
-                context,
-                {'connection_url': self.read_url}
-            )
             new_data_dict = resource_show(context, data_dict)
             try:
-                connection = engine.connect()
+                connection = read_engine.connect()
                 result = connection.execute(
                     'SELECT 1 FROM "_table_metadata" WHERE name = %s AND alias_of IS NULL',
                     new_data_dict['id']
@@ -100,6 +100,24 @@ class DatastorePlugin(p.SingletonPlugin):
             new_resource_show._datastore_wrapped = True
             logic._actions['resource_show'] = new_resource_show
 
+    def notify(self, entity, operation):
+        if not isinstance(entity, model.Package) or self.legacy_mode:
+            return
+        # if a resource is new, it cannot have a datastore resource, yet
+        if operation == model.domain_object.DomainObjectOperation.changed:
+            context = {'model': model, 'ignore_auth': True}
+            if entity.private:
+                func = p.toolkit.get_action('datastore_make_private')
+            else:
+                func = p.toolkit.get_action('datastore_make_public')
+            for resource in entity.resources:
+                try:
+                    func(context, {
+                        'connection_url': self.write_url,
+                        'resource_id': resource.id})
+                except p.toolkit.ObjectNotFound:
+                    pass
+
     def _log_or_raise(self, message):
         if self.config.get('debug'):
             log.critical(message)
@@ -129,10 +147,12 @@ class DatastorePlugin(p.SingletonPlugin):
         ''' Returns True if no connection has CREATE privileges on the public
         schema. This is the case if replication is enabled.'''
         for url in [self.ckan_url, self.write_url, self.read_url]:
-            connection = db._get_engine(None,
-                                        {'connection_url': url}).connect()
-            sql = u"SELECT has_schema_privilege('public', 'CREATE')"
-            is_writable = connection.execute(sql).first()[0]
+            connection = db._get_engine({'connection_url': url}).connect()
+            try:
+                sql = u"SELECT has_schema_privilege('public', 'CREATE')"
+                is_writable = connection.execute(sql).first()[0]
+            finally:
+                connection.close()
             if is_writable:
                 return False
         return True
@@ -148,12 +168,13 @@ class DatastorePlugin(p.SingletonPlugin):
         return self.write_url == self.read_url
 
     def _read_connection_has_correct_privileges(self):
-        ''' Returns True if the right permissions are set for the read only user.
-        A table is created by the write user to test the read only user.
+        ''' Returns True if the right permissions are set for the read
+        only user. A table is created by the write user to test the
+        read only user.
         '''
-        write_connection = db._get_engine(None,
+        write_connection = db._get_engine(
             {'connection_url': self.write_url}).connect()
-        read_connection = db._get_engine(None,
+        read_connection = db._get_engine(
             {'connection_url': self.read_url}).connect()
 
         drop_foo_sql = u'DROP TABLE IF EXISTS _foo'
@@ -161,15 +182,19 @@ class DatastorePlugin(p.SingletonPlugin):
         write_connection.execute(drop_foo_sql)
 
         try:
-            write_connection.execute(u'CREATE TABLE _foo ()')
-            for privilege in ['INSERT', 'UPDATE', 'DELETE']:
-                test_privilege_sql = u"SELECT has_table_privilege('_foo', '{privilege}')"
-                sql = test_privilege_sql.format(privilege=privilege)
-                have_privilege = read_connection.execute(sql).first()[0]
-                if have_privilege:
-                    return False
+            try:
+                write_connection.execute(u'CREATE TABLE _foo ()')
+                for privilege in ['INSERT', 'UPDATE', 'DELETE']:
+                    test_privilege_sql = u"SELECT has_table_privilege('_foo', '{privilege}')"
+                    sql = test_privilege_sql.format(privilege=privilege)
+                    have_privilege = read_connection.execute(sql).first()[0]
+                    if have_privilege:
+                        return False
+            finally:
+                write_connection.execute(drop_foo_sql)
         finally:
-            write_connection.execute(drop_foo_sql)
+            write_connection.close()
+            read_connection.close()
         return True
 
     def _create_alias_table(self):
@@ -193,9 +218,12 @@ class DatastorePlugin(p.SingletonPlugin):
             ORDER BY dependee.oid DESC;
         '''
         create_alias_table_sql = u'CREATE OR REPLACE VIEW "_table_metadata" AS {0}'.format(mapping_sql)
-        connection = db._get_engine(None,
-            {'connection_url': pylons.config['ckan.datastore.write_url']}).connect()
-        connection.execute(create_alias_table_sql)
+        try:
+            connection = db._get_engine(
+                {'connection_url': self.write_url}).connect()
+            connection.execute(create_alias_table_sql)
+        finally:
+            connection.close()
 
     def get_actions(self):
         actions = {'datastore_create': action.datastore_create,
@@ -203,11 +231,21 @@ class DatastorePlugin(p.SingletonPlugin):
                    'datastore_delete': action.datastore_delete,
                    'datastore_search': action.datastore_search}
         if not self.legacy_mode:
-            actions['datastore_search_sql'] = action.datastore_search_sql
+            actions.update({
+                'datastore_search_sql': action.datastore_search_sql,
+                'datastore_make_private': action.datastore_make_private,
+                'datastore_make_public': action.datastore_make_public})
         return actions
 
     def get_auth_functions(self):
         return {'datastore_create': auth.datastore_create,
                 'datastore_upsert': auth.datastore_upsert,
                 'datastore_delete': auth.datastore_delete,
-                'datastore_search': auth.datastore_search}
+                'datastore_search': auth.datastore_search,
+                'datastore_change_permissions': auth.datastore_change_permissions}
+
+    def before_map(self, m):
+        m.connect('/datastore/dump/{resource_id}',
+                  controller='ckanext.datastore.controller:DatastoreController',
+                  action='dump')
+        return m
