@@ -1,6 +1,8 @@
 import json
 import nose
+from nose.tools import assert_equal
 
+import pylons
 import sqlalchemy.orm as orm
 
 import ckan.plugins as p
@@ -25,11 +27,8 @@ class TestDatastoreCreate(tests.WsgiAppCase):
         ctd.CreateTestData.create()
         cls.sysadmin_user = model.User.get('testsysadmin')
         cls.normal_user = model.User.get('annafan')
-        import pylons
         engine = db._get_engine(
-                None,
-                {'connection_url': pylons.config['ckan.datastore.write_url']}
-            )
+            {'connection_url': pylons.config['ckan.datastore.write_url']})
         cls.Session = orm.scoped_session(orm.sessionmaker(bind=engine))
 
     @classmethod
@@ -39,11 +38,10 @@ class TestDatastoreCreate(tests.WsgiAppCase):
 
     @classmethod
     def _configure_iconfigurable_plugins(cls):
-        import pylons.config as config
         from ckan.plugins import PluginImplementations
         from ckan.plugins.interfaces import IConfigurable
         for plugin in PluginImplementations(IConfigurable):
-            plugin.configure(config)
+            plugin.configure(pylons.config)
 
     def test_create_requires_auth(self):
         resource = model.Package.get('annakarenina').resources[0]
@@ -92,6 +90,45 @@ class TestDatastoreCreate(tests.WsgiAppCase):
         res_dict = json.loads(res.body)
         assert res_dict['success'] is False
 
+    def test_create_duplicate_alias_name(self):
+        resource = model.Package.get('annakarenina').resources[0]
+        data = {
+            'resource_id': resource.id,
+            'aliases': u'myalias'
+        }
+        postparams = '%s=1' % json.dumps(data)
+        auth = {'Authorization': str(self.sysadmin_user.apikey)}
+        res = self.app.post('/api/action/datastore_create', params=postparams,
+                            extra_environ=auth, status=200)
+        res_dict = json.loads(res.body)
+        assert res_dict['success'] is True
+
+        # try to create another table with the same alias
+        resource = model.Package.get('annakarenina').resources[1]
+        data = {
+            'resource_id': resource.id,
+            'aliases': u'myalias'
+        }
+        postparams = '%s=1' % json.dumps(data)
+        auth = {'Authorization': str(self.sysadmin_user.apikey)}
+        res = self.app.post('/api/action/datastore_create', params=postparams,
+                            extra_environ=auth, status=409)
+        res_dict = json.loads(res.body)
+        assert res_dict['success'] is False
+
+        # try to create an alias that is a resource id
+        resource = model.Package.get('annakarenina').resources[1]
+        data = {
+            'resource_id': resource.id,
+            'aliases': model.Package.get('annakarenina').resources[0].id
+        }
+        postparams = '%s=1' % json.dumps(data)
+        auth = {'Authorization': str(self.sysadmin_user.apikey)}
+        res = self.app.post('/api/action/datastore_create', params=postparams,
+                            extra_environ=auth, status=409)
+        res_dict = json.loads(res.body)
+        assert res_dict['success'] is False
+
     def test_create_invalid_field_type(self):
         resource = model.Package.get('annakarenina').resources[0]
         data = {
@@ -173,6 +210,7 @@ class TestDatastoreCreate(tests.WsgiAppCase):
         res_dict = json.loads(res.body)
 
         assert res_dict['success'] is False
+        assert_equal(res_dict['error']['__type'], 'Validation Error')
 
         resource = model.Package.get('annakarenina').resources[0]
         data = {
@@ -190,6 +228,7 @@ class TestDatastoreCreate(tests.WsgiAppCase):
         res_dict = json.loads(res.body)
 
         assert res_dict['success'] is False
+        assert_equal(res_dict['error']['__type'], 'Validation Error')
 
     def test_create_invalid_index(self):
         resource = model.Package.get('annakarenina').resources[0]
@@ -258,18 +297,17 @@ class TestDatastoreCreate(tests.WsgiAppCase):
             'fields': [{'id': 'boo%k', 'type': 'text'},
                        {'id': 'author', 'type': 'json'}],
             'indexes': [['boo%k', 'author'], 'author'],
-            'records': [
-                        {'boo%k': 'crime', 'author': ['tolstoy', 'dostoevsky']},
+            'records': [{'boo%k': 'crime', 'author': ['tolstoy', 'dostoevsky']},
                         {'boo%k': 'annakarenina', 'author': ['tolstoy', 'putin']},
                         {'boo%k': 'warandpeace'}]  # treat author as null
         }
-        ### Firstly test to see if resource things it has datastore table
+        ### Firstly test to see whether resource has no datastore table yet
         postparams = '%s=1' % json.dumps({'id': resource.id})
         auth = {'Authorization': str(self.sysadmin_user.apikey)}
         res = self.app.post('/api/action/resource_show', params=postparams,
                             extra_environ=auth)
         res_dict = json.loads(res.body)
-        assert res_dict['result']['datastore_active'] == False
+        assert res_dict['result']['datastore_active'] is False
 
         postparams = '%s=1' % json.dumps(data)
         auth = {'Authorization': str(self.sysadmin_user.apikey)}
@@ -403,7 +441,7 @@ class TestDatastoreCreate(tests.WsgiAppCase):
         res_dict = json.loads(res.body)
 
         assert res_dict['success'] is False
-        assert 'constraints' in res_dict['error']
+        assert 'constraints' in res_dict['error'], res_dict
 
         #######  insert again which should not fail because constraint is removed
         data5 = {
