@@ -2,6 +2,7 @@ import json
 import nose
 import pprint
 
+import pylons
 import sqlalchemy.orm as orm
 
 import ckan.plugins as p
@@ -25,9 +26,10 @@ class TestDatastoreSearch(tests.WsgiAppCase):
         ctd.CreateTestData.create()
         cls.sysadmin_user = model.User.get('testsysadmin')
         cls.normal_user = model.User.get('annafan')
-        resource = model.Package.get('annakarenina').resources[0]
+        cls.dataset = model.Package.get('annakarenina')
+        cls.resource = cls.dataset.resources[0]
         cls.data = {
-            'resource_id': resource.id,
+            'resource_id': cls.resource.id,
             'aliases': 'books3',
             'fields': [{'id': u'b\xfck', 'type': 'text'},
                        {'id': 'author', 'type': 'text'},
@@ -37,7 +39,7 @@ class TestDatastoreSearch(tests.WsgiAppCase):
                         'published': '2005-03-01', 'nested': ['b', {'moo': 'moo'}],
                         u'characters': [u'Princess Anna', u'Sergius']},
                         {u'b\xfck': 'warandpeace', 'author': 'tolstoy',
-                        'nested': {'a':'b'}}
+                        'nested': {'a': 'b'}}
                        ]
         }
         postparams = '%s=1' % json.dumps(cls.data)
@@ -47,6 +49,12 @@ class TestDatastoreSearch(tests.WsgiAppCase):
         res_dict = json.loads(res.body)
         assert res_dict['success'] is True
 
+        # Make an organization, because private datasets must belong to one.
+        cls.organization = tests.call_action_api(
+            cls.app, 'organization_create',
+            name='test_org',
+            apikey=cls.sysadmin_user.apikey)
+
         cls.expected_records = [{u'published': u'2005-03-01T00:00:00',
                                  u'_id': 1,
                                  u'nested': [u'b', {u'moo': u'moo'}],
@@ -60,9 +68,7 @@ class TestDatastoreSearch(tests.WsgiAppCase):
                                  u'author': u'tolstoy',
                                  u'characters': None}]
 
-        import pylons
         engine = db._get_engine(
-                None,
                 {'connection_url': pylons.config['ckan.datastore.write_url']}
             )
         cls.Session = orm.scoped_session(orm.sessionmaker(bind=engine))
@@ -75,7 +81,7 @@ class TestDatastoreSearch(tests.WsgiAppCase):
     def test_search_basic(self):
         data = {'resource_id': self.data['resource_id']}
         postparams = '%s=1' % json.dumps(data)
-        auth = {'Authorization': str(self.sysadmin_user.apikey)}
+        auth = {'Authorization': str(self.normal_user.apikey)}
         res = self.app.post('/api/action/datastore_search', params=postparams,
                             extra_environ=auth)
         res_dict = json.loads(res.body)
@@ -87,7 +93,7 @@ class TestDatastoreSearch(tests.WsgiAppCase):
         # search with parameter id should yield the same results
         data = {'id': self.data['resource_id']}
         postparams = '%s=1' % json.dumps(data)
-        auth = {'Authorization': str(self.sysadmin_user.apikey)}
+        auth = {'Authorization': str(self.normal_user.apikey)}
         res = self.app.post('/api/action/datastore_search', params=postparams,
                             extra_environ=auth)
         res_dict = json.loads(res.body)
@@ -96,10 +102,47 @@ class TestDatastoreSearch(tests.WsgiAppCase):
         assert result['total'] == len(self.data['records'])
         assert result['records'] == self.expected_records, result['records']
 
+    def test_search_private_dataset(self):
+        group = self.dataset.get_groups()[0]
+        context = {
+            'user': self.sysadmin_user.name,
+            'ignore_auth': True,
+            'model': model}
+        package = p.toolkit.get_action('package_create')(
+            context,
+            {'name': 'privatedataset',
+             'private': True,
+             'owner_org' : self.organization['id'],
+             'groups': [{
+                 'id': group.id
+             }]})
+        resource = p.toolkit.get_action('resource_create')(
+            context,
+            {'name': 'privateresource',
+             'url': 'https://www.example.com/',
+             'package_id': package['id']})
+
+        postparams = '%s=1' % json.dumps({
+            'resource_id': resource['id'],
+        })
+        auth = {'Authorization': str(self.sysadmin_user.apikey)}
+        res = self.app.post('/api/action/datastore_create', params=postparams,
+                            extra_environ=auth)
+        res_dict = json.loads(res.body)
+        assert res_dict['success'] is True
+
+        data = {'resource_id': resource['id']}
+        postparams = '%s=1' % json.dumps(data)
+        auth = {'Authorization': str(self.normal_user.apikey)}
+        res = self.app.post('/api/action/datastore_search', params=postparams,
+                            extra_environ=auth, status=403)
+        res_dict = json.loads(res.body)
+        assert res_dict['success'] is False
+
     def test_search_alias(self):
         data = {'resource_id': self.data['aliases']}
         postparams = '%s=1' % json.dumps(data)
-        auth = {'Authorization': str(self.sysadmin_user.apikey)}
+        auth = {'Authorization': str(self.normal_user.apikey)}
         res = self.app.post('/api/action/datastore_search', params=postparams,
                             extra_environ=auth)
         res_dict_alias = json.loads(res.body)
@@ -111,7 +154,7 @@ class TestDatastoreSearch(tests.WsgiAppCase):
         data = {'resource_id': self.data['resource_id'],
                 'fields': [{'id': 'bad'}]}
         postparams = '%s=1' % json.dumps(data)
-        auth = {'Authorization': str(self.sysadmin_user.apikey)}
+        auth = {'Authorization': str(self.normal_user.apikey)}
         res = self.app.post('/api/action/datastore_search', params=postparams,
                             extra_environ=auth, status=409)
         res_dict = json.loads(res.body)
@@ -121,7 +164,7 @@ class TestDatastoreSearch(tests.WsgiAppCase):
         data = {'resource_id': self.data['resource_id'],
                 'fields': [u'b\xfck']}
         postparams = '%s=1' % json.dumps(data)
-        auth = {'Authorization': str(self.sysadmin_user.apikey)}
+        auth = {'Authorization': str(self.normal_user.apikey)}
         res = self.app.post('/api/action/datastore_search', params=postparams,
                             extra_environ=auth)
         res_dict = json.loads(res.body)
@@ -134,7 +177,7 @@ class TestDatastoreSearch(tests.WsgiAppCase):
         data = {'resource_id': self.data['resource_id'],
                 'fields': u'b\xfck, author'}
         postparams = '%s=1' % json.dumps(data)
-        auth = {'Authorization': str(self.sysadmin_user.apikey)}
+        auth = {'Authorization': str(self.normal_user.apikey)}
         res = self.app.post('/api/action/datastore_search', params=postparams,
                             extra_environ=auth)
         res_dict = json.loads(res.body)
@@ -148,7 +191,20 @@ class TestDatastoreSearch(tests.WsgiAppCase):
         data = {'resource_id': self.data['resource_id'],
                 'filters': {u'b\xfck': 'annakarenina'}}
         postparams = '%s=1' % json.dumps(data)
-        auth = {'Authorization': str(self.sysadmin_user.apikey)}
+        auth = {'Authorization': str(self.normal_user.apikey)}
+        res = self.app.post('/api/action/datastore_search', params=postparams,
+                            extra_environ=auth)
+        res_dict = json.loads(res.body)
+        assert res_dict['success'] is True
+        result = res_dict['result']
+        assert result['total'] == 1
+        assert result['records'] == [self.expected_records[0]]
+
+    def test_search_array_filters(self):
+        data = {'resource_id': self.data['resource_id'],
+                'filters': {u'characters': [u'Princess Anna', u'Sergius']}}
+        postparams = '%s=1' % json.dumps(data)
+        auth = {'Authorization': str(self.normal_user.apikey)}
         res = self.app.post('/api/action/datastore_search', params=postparams,
                             extra_environ=auth)
         res_dict = json.loads(res.body)
@@ -167,24 +223,22 @@ class TestDatastoreSearch(tests.WsgiAppCase):
         assert result['total'] == 1
         assert result['records'] == [self.expected_records[0]]
 
-    def test_search_array_filters(self):
+    def test_search_invalid_filter(self):
         data = {'resource_id': self.data['resource_id'],
-                'filters': {u'characters': [u'Princess Anna', u'Sergius']}}
+                # invalid because author is not an array
+                'filters': {u'author': [u'tolstoy']}}
         postparams = '%s=1' % json.dumps(data)
         auth = {'Authorization': str(self.sysadmin_user.apikey)}
         res = self.app.post('/api/action/datastore_search', params=postparams,
-                            extra_environ=auth)
+                            extra_environ=auth, status=409)
         res_dict = json.loads(res.body)
-        assert res_dict['success'] is True
-        result = res_dict['result']
-        assert result['total'] == 1
-        assert result['records'] == [self.expected_records[0]]
+        assert res_dict['success'] is False
 
     def test_search_sort(self):
         data = {'resource_id': self.data['resource_id'],
                 'sort': u'b\xfck asc, author desc'}
         postparams = '%s=1' % json.dumps(data)
-        auth = {'Authorization': str(self.sysadmin_user.apikey)}
+        auth = {'Authorization': str(self.normal_user.apikey)}
         res = self.app.post('/api/action/datastore_search', params=postparams,
                             extra_environ=auth)
         res_dict = json.loads(res.body)
@@ -206,11 +260,22 @@ class TestDatastoreSearch(tests.WsgiAppCase):
 
         assert result['records'] == self.expected_records[::-1]
 
+    def test_search_invalid(self):
+        data = {'resource_id': self.data['resource_id'],
+                'sort': u'f\xfc\xfc asc'}
+        postparams = '%s=1' % json.dumps(data)
+        auth = {'Authorization': str(self.sysadmin_user.apikey)}
+        res = self.app.post('/api/action/datastore_search', params=postparams,
+                            extra_environ=auth, status=409)
+        res_dict = json.loads(res.body)
+        assert res_dict['success'] is False
+        assert res_dict['error']['sort'][0] == u'field "f\xfc\xfc" not in table'
+
     def test_search_limit(self):
         data = {'resource_id': self.data['resource_id'],
                 'limit': 1}
         postparams = '%s=1' % json.dumps(data)
-        auth = {'Authorization': str(self.sysadmin_user.apikey)}
+        auth = {'Authorization': str(self.normal_user.apikey)}
         res = self.app.post('/api/action/datastore_search', params=postparams,
                             extra_environ=auth)
         res_dict = json.loads(res.body)
@@ -223,7 +288,7 @@ class TestDatastoreSearch(tests.WsgiAppCase):
         data = {'resource_id': self.data['resource_id'],
                 'limit': 'bad'}
         postparams = '%s=1' % json.dumps(data)
-        auth = {'Authorization': str(self.sysadmin_user.apikey)}
+        auth = {'Authorization': str(self.normal_user.apikey)}
         res = self.app.post('/api/action/datastore_search', params=postparams,
                             extra_environ=auth, status=409)
         res_dict = json.loads(res.body)
@@ -232,7 +297,7 @@ class TestDatastoreSearch(tests.WsgiAppCase):
         data = {'resource_id': self.data['resource_id'],
                 'limit': -1}
         postparams = '%s=1' % json.dumps(data)
-        auth = {'Authorization': str(self.sysadmin_user.apikey)}
+        auth = {'Authorization': str(self.normal_user.apikey)}
         res = self.app.post('/api/action/datastore_search', params=postparams,
                             extra_environ=auth, status=409)
         res_dict = json.loads(res.body)
@@ -243,7 +308,7 @@ class TestDatastoreSearch(tests.WsgiAppCase):
                 'limit': 1,
                 'offset': 1}
         postparams = '%s=1' % json.dumps(data)
-        auth = {'Authorization': str(self.sysadmin_user.apikey)}
+        auth = {'Authorization': str(self.normal_user.apikey)}
         res = self.app.post('/api/action/datastore_search', params=postparams,
                             extra_environ=auth)
         res_dict = json.loads(res.body)
@@ -256,7 +321,7 @@ class TestDatastoreSearch(tests.WsgiAppCase):
         data = {'resource_id': self.data['resource_id'],
                 'offset': 'bad'}
         postparams = '%s=1' % json.dumps(data)
-        auth = {'Authorization': str(self.sysadmin_user.apikey)}
+        auth = {'Authorization': str(self.normal_user.apikey)}
         res = self.app.post('/api/action/datastore_search', params=postparams,
                             extra_environ=auth, status=409)
         res_dict = json.loads(res.body)
@@ -265,7 +330,7 @@ class TestDatastoreSearch(tests.WsgiAppCase):
         data = {'resource_id': self.data['resource_id'],
                 'offset': -1}
         postparams = '%s=1' % json.dumps(data)
-        auth = {'Authorization': str(self.sysadmin_user.apikey)}
+        auth = {'Authorization': str(self.normal_user.apikey)}
         res = self.app.post('/api/action/datastore_search', params=postparams,
                             extra_environ=auth, status=409)
         res_dict = json.loads(res.body)
@@ -276,7 +341,7 @@ class TestDatastoreSearch(tests.WsgiAppCase):
                 'q': 'annakarenina'}
 
         postparams = '%s=1' % json.dumps(data)
-        auth = {'Authorization': str(self.sysadmin_user.apikey)}
+        auth = {'Authorization': str(self.normal_user.apikey)}
         res = self.app.post('/api/action/datastore_search', params=postparams,
                             extra_environ=auth)
         res_dict = json.loads(res.body)
@@ -335,7 +400,7 @@ class TestDatastoreSearch(tests.WsgiAppCase):
     def test_search_table_metadata(self):
         data = {'resource_id': "_table_metadata"}
         postparams = '%s=1' % json.dumps(data)
-        auth = {'Authorization': str(self.sysadmin_user.apikey)}
+        auth = {'Authorization': str(self.normal_user.apikey)}
         res = self.app.post('/api/action/datastore_search', params=postparams,
                             extra_environ=auth)
         res_dict = json.loads(res.body)
@@ -375,7 +440,7 @@ class TestDatastoreFullTextSearch(tests.WsgiAppCase):
             ]
         )
         postparams = '%s=1' % json.dumps(cls.data)
-        auth = {'Authorization': str(cls.sysadmin_user.apikey)}
+        auth = {'Authorization': str(cls.normal_user.apikey)}
         res = cls.app.post('/api/action/datastore_create', params=postparams,
                            extra_environ=auth)
         res_dict = json.loads(res.body)
@@ -390,7 +455,7 @@ class TestDatastoreFullTextSearch(tests.WsgiAppCase):
         data = {'resource_id': self.data['resource_id'],
                 'q': 'DE'}
         postparams = '%s=1' % json.dumps(data)
-        auth = {'Authorization': str(self.sysadmin_user.apikey)}
+        auth = {'Authorization': str(self.normal_user.apikey)}
         res = self.app.post('/api/action/datastore_search', params=postparams,
                             extra_environ=auth)
         res_dict = json.loads(res.body)
@@ -401,7 +466,7 @@ class TestDatastoreFullTextSearch(tests.WsgiAppCase):
                 'plain': 'False',
                 'q': 'DE | UK'}
         postparams = '%s=1' % json.dumps(data)
-        auth = {'Authorization': str(self.sysadmin_user.apikey)}
+        auth = {'Authorization': str(self.normal_user.apikey)}
         res = self.app.post('/api/action/datastore_search', params=postparams,
                             extra_environ=auth)
         res_dict = json.loads(res.body)
@@ -417,12 +482,14 @@ class TestDatastoreSQL(tests.WsgiAppCase):
         if not tests.is_datastore_supported():
             raise nose.SkipTest("Datastore not supported")
         plugin = p.load('datastore')
+        plugin.configure(pylons.config)
         if plugin.legacy_mode:
             raise nose.SkipTest("SQL tests are not supported in legacy mode")
         ctd.CreateTestData.create()
         cls.sysadmin_user = model.User.get('testsysadmin')
         cls.normal_user = model.User.get('annafan')
-        resource = model.Package.get('annakarenina').resources[0]
+        cls.dataset = model.Package.get('annakarenina')
+        resource = cls.dataset.resources[0]
         cls.data = {
             'resource_id': resource.id,
             'aliases': 'books4',
@@ -435,8 +502,7 @@ class TestDatastoreSQL(tests.WsgiAppCase):
                         'nested': ['b', {'moo': 'moo'}]},
                         {u'b\xfck': 'warandpeace',
                         'author': 'tolstoy',
-                        'nested': {'a':'b'}}
-                       ]
+                        'nested': {'a': 'b'}}]
         }
         postparams = '%s=1' % json.dumps(cls.data)
         auth = {'Authorization': str(cls.sysadmin_user.apikey)}
@@ -444,24 +510,34 @@ class TestDatastoreSQL(tests.WsgiAppCase):
                            extra_environ=auth)
         res_dict = json.loads(res.body)
         assert res_dict['success'] is True
+        
+        # Make an organization, because private datasets must belong to one.
+        cls.organization = tests.call_action_api(
+            cls.app, 'organization_create',
+            name='test_org',
+            apikey=cls.sysadmin_user.apikey)
 
         cls.expected_records = [{u'_full_text': u"'annakarenina':1 'b':3 'moo':4 'tolstoy':2",
-                                  u'_id': 1,
-                                  u'author': u'tolstoy',
-                                  u'b\xfck': u'annakarenina',
-                                  u'nested': [u'b', {u'moo': u'moo'}],
-                                  u'published': u'2005-03-01T00:00:00'},
-                                 {u'_full_text': u"'b':3 'tolstoy':2 'warandpeac':1",
-                                  u'_id': 2,
-                                  u'author': u'tolstoy',
-                                  u'b\xfck': u'warandpeace',
-                                  u'nested': {u'a': u'b'},
-                                  u'published': None}]
+                                 u'_id': 1,
+                                 u'author': u'tolstoy',
+                                 u'b\xfck': u'annakarenina',
+                                 u'nested': [u'b', {u'moo': u'moo'}],
+                                 u'published': u'2005-03-01T00:00:00'},
+                                {u'_full_text': u"'b':3 'tolstoy':2 'warandpeac':1",
+                                 u'_id': 2,
+                                 u'author': u'tolstoy',
+                                 u'b\xfck': u'warandpeace',
+                                 u'nested': {u'a': u'b'},
+                                 u'published': None}]
         cls.expected_join_results = [{u'first': 1, u'second': 1}, {u'first': 1, u'second': 2}]
 
+        engine = db._get_engine(
+            {'connection_url': pylons.config['ckan.datastore.write_url']})
+        cls.Session = orm.scoped_session(orm.sessionmaker(bind=engine))
+
     @classmethod
     def teardown_class(cls):
-        model.repo.rebuild_db()
+        rebuild_all_dbs(cls.Session)
         p.unload('datastore')
 
     def test_is_single_statement(self):
@@ -481,20 +557,20 @@ class TestDatastoreSQL(tests.WsgiAppCase):
             assert db._is_single_statement(multiple) is False
 
     def test_invalid_statement(self):
-        query = 'SELECT ** FROM public.foobar'
+        query = 'SELECT ** FROM foobar'
         data = {'sql': query}
         postparams = json.dumps(data)
-        auth = {'Authorization': str(self.sysadmin_user.apikey)}
+        auth = {'Authorization': str(self.normal_user.apikey)}
         res = self.app.post('/api/action/datastore_search_sql', params=postparams,
                             extra_environ=auth, status=409)
         res_dict = json.loads(res.body)
         assert res_dict['success'] is False
 
     def test_select_basic(self):
-        query = 'SELECT * FROM public."{0}"'.format(self.data['resource_id'])
+        query = 'SELECT * FROM "{0}"'.format(self.data['resource_id'])
         data = {'sql': query}
         postparams = json.dumps(data)
-        auth = {'Authorization': str(self.sysadmin_user.apikey)}
+        auth = {'Authorization': str(self.normal_user.apikey)}
         res = self.app.post('/api/action/datastore_search_sql', params=postparams,
                             extra_environ=auth)
         res_dict = json.loads(res.body)
@@ -503,7 +579,7 @@ class TestDatastoreSQL(tests.WsgiAppCase):
         assert result['records'] == self.expected_records
 
         # test alias search
-        query = 'SELECT * FROM public."{0}"'.format(self.data['aliases'])
+        query = 'SELECT * FROM "{0}"'.format(self.data['aliases'])
         data = {'sql': query}
         postparams = json.dumps(data)
         res = self.app.post('/api/action/datastore_search_sql', params=postparams,
@@ -534,10 +610,135 @@ class TestDatastoreSQL(tests.WsgiAppCase):
             '''.format(self.data['resource_id'])
         data = {'sql': query}
         postparams = json.dumps(data)
-        auth = {'Authorization': str(self.sysadmin_user.apikey)}
+        auth = {'Authorization': str(self.normal_user.apikey)}
         res = self.app.post('/api/action/datastore_search_sql', params=postparams,
                             extra_environ=auth)
         res_dict = json.loads(res.body)
         assert res_dict['success'] is True
         result = res_dict['result']
         assert result['records'] == self.expected_join_results
+
+    def test_read_private(self):
+        context = {
+            'user': self.sysadmin_user.name,
+            'model': model}
+        data_dict = {
+            'resource_id': self.data['resource_id'],
+            'connection_url': pylons.config['ckan.datastore.write_url']}
+        p.toolkit.get_action('datastore_make_private')(context, data_dict)
+        query = 'SELECT * FROM "{0}"'.format(self.data['resource_id'])
+        data = {'sql': query}
+        postparams = json.dumps(data)
+        auth = {'Authorization': str(self.normal_user.apikey)}
+        res = self.app.post('/api/action/datastore_search_sql', params=postparams,
+                            extra_environ=auth, status=403)
+        res_dict = json.loads(res.body)
+        assert res_dict['success'] is False
+        assert res_dict['error']['__type'] == 'Authorization Error'
+
+        # make it public for the other tests
+        p.toolkit.get_action('datastore_make_public')(context, data_dict)
+
+    def test_new_datastore_table_from_private_resource(self):
+        # make a private CKAN resource
+        group = self.dataset.get_groups()[0]
+        context = {
+            'user': self.sysadmin_user.name,
+            'ignore_auth': True,
+            'model': model}
+        package = p.toolkit.get_action('package_create')(
+            context,
+            {'name': 'privatedataset',
+             'private': True,
+             'owner_org': self.organization['id'],
+             'groups': [{
+                 'id': group.id
+             }]})
+        resource = p.toolkit.get_action('resource_create')(
+            context,
+            {'name': 'privateresource',
+             'url': 'https://www.example.com/',
+             'package_id': package['id']})
+
+        postparams = '%s=1' % json.dumps({
+            'resource_id': resource['id'],
+        })
+        auth = {'Authorization': str(self.sysadmin_user.apikey)}
+        res = self.app.post('/api/action/datastore_create', params=postparams,
+                            extra_environ=auth)
+        res_dict = json.loads(res.body)
+        assert res_dict['success'] is True
+
+        # new resource should be private
+        query = 'SELECT * FROM "{0}"'.format(resource['id'])
+        data = {'sql': query}
+        postparams = json.dumps(data)
+        auth = {'Authorization': str(self.normal_user.apikey)}
+        res = self.app.post('/api/action/datastore_search_sql', params=postparams,
+                            extra_environ=auth, status=403)
+        res_dict = json.loads(res.body)
+        assert res_dict['success'] is False
+        assert res_dict['error']['__type'] == 'Authorization Error'
+
+    def test_making_resource_private_makes_datastore_private(self):
+        group = self.dataset.get_groups()[0]
+        context = {
+            'user': self.sysadmin_user.name,
+            'ignore_auth': True,
+            'model': model}
+        package = p.toolkit.get_action('package_create')(
+            context,
+            {'name': 'privatedataset2',
+             'private': False,
+             'owner_org': self.organization['id'],
+             'groups': [{
+                 'id': group.id
+             }]})
+        resource = p.toolkit.get_action('resource_create')(
+            context,
+            {'name': 'privateresource2',
+             'url': 'https://www.example.co.uk/',
+             'package_id': package['id']})
+
+        postparams = '%s=1' % json.dumps({
+            'resource_id': resource['id']})
+        auth = {'Authorization': str(self.sysadmin_user.apikey)}
+        res = self.app.post('/api/action/datastore_create', params=postparams,
+                            extra_environ=auth)
+        res_dict = json.loads(res.body)
+        assert res_dict['success'] is True
+
+        # Test public resource
+        query = 'SELECT * FROM "{0}"'.format(resource['id'])
+        data = {'sql': query}
+        postparams = json.dumps(data)
+        auth = {'Authorization': str(self.normal_user.apikey)}
+        res = self.app.post('/api/action/datastore_search_sql', params=postparams,
+                            extra_environ=auth)
+        res_dict = json.loads(res.body)
+        assert res_dict['success'] is True
+
+        # Make resource private
+        package = p.toolkit.get_action('package_show')(
+            context, {'id': package.get('id')})
+        package['private'] = True
+        package = p.toolkit.get_action('package_update')(context, package)
+
+        # Test private
+        res = self.app.post('/api/action/datastore_search_sql', params=postparams,
+                            extra_environ=auth, status=403)
+        res_dict = json.loads(res.body)
+        assert res_dict['success'] is False
+        assert res_dict['error']['__type'] == 'Authorization Error'
+
+        # Make resource public
+        package = p.toolkit.get_action('package_show')(
+            context, {'id': package.get('id')})
+        package['private'] = False
+        package = p.toolkit.get_action('package_update')(context, package)
+
+        # Test public again
+        res = self.app.post('/api/action/datastore_search_sql', params=postparams,
+                            extra_environ=auth)
+        res_dict = json.loads(res.body)
+        assert res_dict['success'] is True
